from flask import Flask, render_template, request, redirect, url_for, jsonify, flash
from data_preparation.data_preparation import get_most_nested_element
import os
import json
import random
import pandas as pd
from config import secret_key, data_path_captcha, captcha_true_min, captcha_true_max, data_path_logs

app = Flask(__name__)
app.secret_key = secret_key


def choose_images(data_path, minimum_pictures, maximum_pictures):
    chosen_images = []
    # Cyklem vybíráme náhodný počet obrázků v rozmezí
    for _ in range(random.randint(minimum_pictures, maximum_pictures)):
        while True:
            chosen_image = random.choice(os.listdir(data_path))
            if chosen_image not in chosen_images:
                chosen_images.append(chosen_image)
                break
            else:
                continue
    full_path_chosen_images = [os.path.join(data_path, image_name) for image_name in chosen_images]
    return full_path_chosen_images


# Výběr obrázků do CAPTCHA
def generate_captcha():
    # Vybereme jednu ze složek, kde máme data
    chosen_folder = os.path.join(data_path_captcha, random.choice(os.listdir(data_path_captcha)))
    chosen_folder_name = get_most_nested_element(chosen_folder)
    true_images_folder = os.path.join(chosen_folder, "true")
    unknown_images_folder = os.path.join(chosen_folder, "unknown")
    # Zavoláme funkci pro výběr obrázků
    chosen_true_images = choose_images(true_images_folder, captcha_true_min, captcha_true_max)
    chosen_unknown_images = choose_images(unknown_images_folder, 9 - len(chosen_true_images),
                                          9 - len(chosen_true_images))

    return chosen_true_images, chosen_unknown_images, chosen_folder_name


# Generování CAPTCHA do HTML
def generate_html_captcha():
    # Necháme generovat CAPTCHA
    true_images, unknown_images, chosen_folder_name = generate_captcha()
    # Obrázky spojíme
    images_for_captcha = true_images + unknown_images
    # Promícháme
    random.shuffle(images_for_captcha)
    # Sestojíme HTML část, která bnude doplněna do template
    captcha_div = f"<p>Vyberte obrázky obsahující {chosen_folder_name}</p>\n<div class='captcha-container'>"
    for index, image in enumerate(images_for_captcha, start=1):
        div_for_html = f"\n<div class='captcha-item'>\n <img src='{image}' alt='Image{index}'> \n </div>"
        captcha_div += div_for_html

    captcha_div += "\n </div>"
    return captcha_div, true_images, unknown_images, chosen_folder_name


# Uložení jsonu
def save_results_to_file(results, filename):
    with open(filename, 'w') as file:
        json.dump(results, file)


# Načtení jsonu
def load_results_from_file(filename):
    if os.path.exists(filename):
        with open(filename, 'r') as file:
            return json.load(file)
    else:
        return {'results': []}


# Logování výsledků
def log_result(folder_name, status, true_images, unknown_images, selected_images):
    true_images = [get_most_nested_element(image_path) for image_path in true_images]
    unknown_images = [get_most_nested_element(image_path) for image_path in unknown_images]
    selected_images = [get_most_nested_element(image_path) for image_path in selected_images]
    existing_results = load_results_from_file(os.path.join(data_path_logs, 'results.json'))
    # Výběr logovaných infromací a struktura
    existing_results['results'].append(
        {'time': pd.Timestamp.now().strftime('%Y-%m-%d %H:%M:%S'), 'folder_name': folder_name, 'status': status,
         'true_images': true_images, 'unknown_images': unknown_images, 'selected_images': selected_images})
    # Uložení
    save_results_to_file(existing_results, os.path.join(data_path_logs, 'results.json'))


# Flask
@app.route('/', methods=['GET', 'POST'])
def index():
    global true_images, unknown_images, chosen_folder_name
    # Generování CAPTCHA a předložení uživateli
    if request.method == 'GET':
        captcha, true_images, unknown_images, chosen_folder_name = generate_html_captcha()
        return render_template('index.html', captcha=captcha)

    # Příjem odpovědi od uživatele
    elif request.method == 'POST':
        selected_images = request.form.get('selected_images')
        try:
            selected_images = json.loads(selected_images)
            # Kontrola správné odpovědi a logování odpovědi
            correct_selection = all(image_path in selected_images for image_path in true_images)
            if correct_selection:
                log_result(chosen_folder_name, 'success', true_images, unknown_images, selected_images)
                return jsonify({'status': 'success', 'message': 'Vybrány správné obrázky!'})
            else:
                log_result(chosen_folder_name, 'error', true_images, unknown_images, selected_images)
                # Posíláme uživateli hlášku, že odpověděl nesprávně
                flash('Vybrány nesprávné obrázky! Zkuste to znovu.', 'error')
                return redirect(url_for(".index"))

        except Exception as e:
            print(e)
            return redirect(url_for(".index"))


# Spuštění flask aplikace
if __name__ == '__main__':
    app.run(debug=True)
